/*-
 * Copyright (c) 2017-2019 Carsten Sonne Larsen <cs@innolan.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "config.h"
#include "message.h"
#include "mem.h"

static void SendTextMessage(char *text, long type)
{
    Forbid();
    if (!Globals->ShuttingDown)
    {
        struct AppWindowMessage *message = (struct AppWindowMessage *)AllocMemSafe(sizeof(struct AppWindowMessage));
        message->Msg.mn_Node.ln_Type = NT_MESSAGE;
        message->Msg.mn_Length = sizeof(struct AppWindowMessage);
        message->Msg.mn_ReplyPort = Globals->Broker->ReplyPort;
        message->Type = type;
        message->Text = StrDupSafe(text);
        PutMsg(Globals->Broker->UserPort, (struct Message *)message);
    }
    Permit();
}

void SendErrorMessage(char *text)
{
    SendTextMessage(text, ATK_LOGERROR);
}

void SendWarningMessage(char *text)
{
    SendTextMessage(text, ATK_LOGWARN);
}

void SendInfoMessage(char *text)
{
    SendTextMessage(text, ATK_LOGINFO);
}

void SendTraceMessage(char *text)
{
    SendTextMessage(text, ATK_LOGTRACE);
}

void SendMessage(long type)
{
    Forbid();
    if (!Globals->ShuttingDown)
    {
        struct AppWindowMessage *message = (struct AppWindowMessage *)AllocMemSafe(sizeof(struct AppWindowMessage));
        message->Msg.mn_Node.ln_Type = NT_MESSAGE;
        message->Msg.mn_Length = sizeof(struct AppWindowMessage);
        message->Msg.mn_ReplyPort = Globals->Broker->ReplyPort;
        message->Type = type;
        message->Text = NULL;
        PutMsg(Globals->Broker->UserPort, (struct Message *)message);
    }
    Permit();
}

void SendMessageTo(struct MsgPort *dest, struct MsgPort *reply, long type)
{
    Forbid();
    if (!Globals->ShuttingDown)
    {
        struct AppWindowMessage *message = (struct AppWindowMessage *)AllocMemSafe(sizeof(struct AppWindowMessage));
        message->Msg.mn_Node.ln_Type = NT_MESSAGE;
        message->Msg.mn_Length = sizeof(struct AppWindowMessage);
        message->Msg.mn_ReplyPort = reply;
        message->Type = type;
        message->Text = NULL;
        PutMsg(dest, (struct Message *)message);
    }
    Permit();
}
