/*-
 * Copyright (c) 2017-2019 Carsten Sonne Larsen <cs@innolan.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "config.h"

#if !defined(HAVE_POLL)
#if defined(__CLIB2__)
#include <sys/select.h>
#endif

int poll(struct pollfd *fds, nfds_t nfds, int timeout)
{
    unsigned int i;
    int maxfds = -1, ret;
    fd_set readSet, writeSet, errorSet;
    struct timeval timeout_tv, *tvp = NULL;

    if (timeout >= 0)
    {
        timeout_tv.tv_secs = (timeout / 1000);
        timeout_tv.tv_micro = (timeout % 1000) * 1000;
        tvp = &timeout_tv;
    }

    FD_ZERO(&readSet);
    FD_ZERO(&writeSet);
    FD_ZERO(&errorSet);

    for (i = 0; i < nfds; i++)
    {
        fds[i].revents = 0;

        if (fds[i].events == 0)
            continue;

        if (fds[i].fd > maxfds)
            maxfds = fds[i].fd;

        if (fds[i].events & POLLIN)
            FD_SET(fds[i].fd, &readSet);
    }

    ret = WaitSelect(maxfds + 1, &readSet, &writeSet, &errorSet, tvp, NULL);

    if (ret == -1)
        return ret;

    for (i = 0; i < nfds; i++)
    {
        if (fds[i].events == 0)
            continue;

        if (FD_ISSET(fds[i].fd, &readSet))
            fds[i].revents |= POLLIN;
    }

    return ret;
}
#endif