/*-
 * Copyright (c) 2017-2019 Carsten Sonne Larsen <cs@innolan.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef STATE_H_INCLUDED
#define STATE_H_INCLUDED

#include "config.h"

#define PrioritySet 0x001
#define VerboseSet 0x002
#define TimeoutSet 0x004
#define IntervalSet 0x008
#define ReadonlySet 0x010
#define DestinationAddressSet 0x020
#define DestinationPortSet 0x040
#define ThresholdSet 0x080
#define LogFileSet 0x200

#define DefaultSettingType 0x01
#define FileSettingType 0x02
#define CliSettingType 0x03
#define WbSettingType 0x04
#define GlobalSettingType 0x10

struct AppBroker
{
    CxObj *Object;
    short ShutdownSigBit;
    struct Task *Task;
    struct MsgPort *BrokerPort;
    struct MsgPort *UserPort;
    struct MsgPort *ReplyPort;
};

struct AppCom;
struct AppSettingWindowGadgets;

struct AppSettingWindow
{
    struct Process *Task;
    struct Screen *Screen;
    struct Window *SettingWindow;
    struct Gadget *GadgetList;
    struct MsgPort *UserPort;
    void *VisualInfo;
    char *PriorityText;
    char *ThresholdText;
    char *TimeZoneText;
    long Width;
    long Height;
    struct AppSettings *Settings;
    struct AppSettingWindowGadgets *Gadgets;
};

struct AppSettings
{
    long Priority;
    long Verbose;
    long Timeout;
    long Interval;
    long Readonly;
    long long Threshold;
    char *DestinationAddress;
    char *DestinationPort;
    char *LogFile;
    long Values;
    long Type;
};

struct AppState
{
    bool Active;
    bool ShuttingDown;
    struct timeval LastNtpSync;
    struct Locale *Locale;
    struct AppCom *Syncer;
    struct AppBroker *Broker;
    struct AppSettingWindow *Window;
    struct AppSignals *Signals;
    struct AppSettings *Settings;
};

struct AppSettingKeys
{
    const char *Priority;
    const char *Verbose;
    const char *Timeout;
    const char *Interval;
    const char *Readonly;
    const char *DestinationAddress;
    const char *DestinationPort;
    const char *Threshold;
    const char *LogFile;
};

typedef void (*ParseSettingFunction)(struct AppSettings *, void *);

struct SettingFunc
{
    const char *Name;
    ParseSettingFunction Function;
};

extern struct AppState *Globals;
extern struct AppSettingKeys *SettingKeys;
extern const struct SettingFunc settingFunctions[];
extern const struct AppSettings DefaultSettings;

void InitState(void);
void DestroyState(void);
void SanitizeSettings(void);
void ShowSettings(void);
void LoadSettings(void);
void SaveSettings(bool);

struct AppSettings *CreateSettings(long);
struct AppSettings *CopySettings(const struct AppSettings *);
void ApplyAppSettings(struct AppSettings *, bool);
void FreeSettings(struct AppSettings *);
void CacheSettings(struct AppSettings *);
void ApplySettings(void);

#endif
